/** /Context/Query
 *
 * @author Alex Malotky
 */
import { D1Database } from "@cloudflare/workers-types";
import ObjectValidator, { ObjectProperties, ObjectDefaults } from "../Validation/Object";
import { Object } from "../Validation/Type";
import { TypeOf } from "../Validation";
/** Database Query Builder
 *
 * Currntly only works with Cloudflare D1Database
 */
export default class Query {
    #private;
    private _table;
    constructor(table: string, db: D1Database);
    /** Insert Value into Database
     *
     * @param {ObjectValidator} validator
     * @param {Object} value
     */
    insert<P extends ObjectProperties>(validator: ObjectValidator<P>, value: Object<keyof P>): Promise<void>;
    /** Update Value in Database
     *
     * @param {ObjectValidator} validator
     * @param {Object} value
     * @param {ObjectDefaults} constraints
     */
    update<P extends ObjectProperties>(validator: ObjectValidator<P>, value: Object<keyof P>, constraints?: ObjectDefaults<keyof P>): Promise<void>;
    /** Get First Value from Database
     *
     * @param {ObjectValidator} validator
     * @param {ObjectDefaults} constraints
     */
    get<P extends ObjectProperties>(validator: ObjectValidator<P>, constraints?: ObjectDefaults<keyof P>): Promise<TypeOf<ObjectValidator<P>> | null>;
    /** Get ALl Values from Database
     *
     * @param {ObjectValidator} validator
     * @param {ObjectDefaults} constraints
     */
    getAll<P extends ObjectProperties>(validator: ObjectValidator<P>, constraints?: ObjectDefaults<keyof P>): Promise<TypeOf<ObjectValidator<P>>[]>;
}
