"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Query_db;
Object.defineProperty(exports, "__esModule", { value: true });
/** Database Query Builder
 *
 * Currntly only works with Cloudflare D1Database
 */
class Query {
    constructor(table, db) {
        _Query_db.set(this, void 0);
        __classPrivateFieldSet(this, _Query_db, db, "f");
        this._table = table;
    }
    /** Insert Value into Database
     *
     * @param {ObjectValidator} validator
     * @param {Object} value
     */
    async insert(validator, value) {
        let queryNames = this._table + "(";
        let queryValues = "VALUES(";
        const values = [];
        for (const name in value) {
            queryNames += name + ", ";
            queryValues += "?, ";
            values.push(validator.get(name).simplify(value[name]));
        }
        await __classPrivateFieldGet(this, _Query_db, "f").prepare(`INSERT INTO ${this._table}${queryNames.slice(-2)}) ${queryValues.slice(-2)})`)
            .bind(...values).run();
    }
    /** Update Value in Database
     *
     * @param {ObjectValidator} validator
     * @param {Object} value
     * @param {ObjectDefaults} constraints
     */
    async update(validator, value, constraints) {
        let queryNames = "SET ";
        const values = [];
        for (const name in value) {
            queryNames += name + " = ?, ";
            values.push(validator.get(name).simplify(value[name]));
        }
        let constraintNames = "";
        if (constraints) {
            constraintNames = "WHERE ";
            for (const name in constraints) {
                constraintNames += name + " = ? AND";
                values.push(validator.get(name).simplify(constraints[name]));
            }
            constraintNames = constraintNames.slice(-3);
        }
        await __classPrivateFieldGet(this, _Query_db, "f").prepare(`UPDATE ${this._table} ${queryNames} ${constraintNames}`)
            .bind(...values).run();
    }
    /** Get First Value from Database
     *
     * @param {ObjectValidator} validator
     * @param {ObjectDefaults} constraints
     */
    async get(validator, constraints) {
        let constraintNames = "";
        const values = [];
        if (constraints) {
            constraintNames = "WHERE ";
            for (const name in constraints) {
                constraintNames += name + " = ? AND";
                values.push(validator.get(name).simplify(constraints[name]));
            }
            constraintNames = constraintNames.slice(-3);
        }
        const result = await __classPrivateFieldGet(this, _Query_db, "f").prepare(`SELECT * FROM ${this._table} ${constraintNames}`)
            .bind(...values).first();
        if (result === null)
            return null;
        for (const name in result) {
            result[name] = validator.get(name).run(result[name]);
        }
        return result;
    }
    /** Get ALl Values from Database
     *
     * @param {ObjectValidator} validator
     * @param {ObjectDefaults} constraints
     */
    async getAll(validator, constraints) {
        let constraintNames = "";
        const values = [];
        if (constraints) {
            constraintNames = "WHERE ";
            for (const name in constraints) {
                constraintNames += name + " = ? AND";
                values.push(validator.get(name).simplify(constraints[name]));
            }
            constraintNames = constraintNames.slice(-3);
        }
        const output = [];
        const { results, error } = await __classPrivateFieldGet(this, _Query_db, "f").prepare(`SELECT * FROM ${this._table} ${constraintNames}`)
            .bind(...values).all();
        if (error)
            throw error;
        for (const result of results) {
            for (const name in result) {
                result[name] = validator.get(name).run(result[name]);
            }
            output.push(result);
        }
        return output;
    }
}
_Query_db = new WeakMap();
exports.default = Query;
