/** /Validation
 *
 * @author Alex Malotky
 */
import Type, { Color, Date, DateTime, Email, Telephone, Time, Url, TypeValidator, List } from "./Type";
import BooleanValidator from "./Boolean";
import NumberValidator from "./Number";
import StringValidator from "./String";
import ColorValidator from "./Color";
import DateValidator from "./Date";
import DateTimeValidator from "./DateTime";
import EmailValidator from "./Email";
import FileValidator from "./File";
import TelephoneValidator from "./Telephone";
import TimeValidator from "./Time";
import UrlValidator from "./Url";
import EmptyValidator from "./Empty";
import ListValidator from "./List";
import ObjectValidator, { ObjectProperties, ObjectDefaults } from "./Object";
import OptionalValidator from "./Optional";
declare const _default: {
    boolean: (defaultValue?: boolean) => BooleanValidator;
    number: (defaultValue?: number) => NumberValidator;
    string: (defaultValue?: string) => StringValidator;
    Color: (defaultValue?: Color) => ColorValidator;
    Date: (defaultValue?: Date) => DateValidator;
    DateTime: (defaultValue?: DateTime) => DateTimeValidator;
    Email: (defaultValue?: Email) => EmailValidator;
    Telephone: (defaultValue?: Telephone) => TelephoneValidator;
    Time: (defaultValue?: Time) => TimeValidator;
    Url: (defaultValue?: Url) => UrlValidator;
    Empty: () => EmptyValidator;
    File: () => FileValidator;
    List: <T extends Type>(type: TypeValidator<T>, seperator?: string, defaultValue?: List<T>) => ListValidator<T>;
    Object: <P extends ObjectProperties>(properties: P, defaultValue?: ObjectDefaults<keyof P>) => ObjectValidator<P>;
    Optional: <T_1 extends Type>(type: TypeValidator<T_1>, defaultValue?: T_1 | null | undefined) => OptionalValidator<T_1>;
};
export default _default;
export type TypeOf<T extends TypeValidator<Type>> = T["_type"];
/** Basic Types Map
 *
 * Can hopefully use this to create a function that takes easier to read objects.
 */
